
// sampleDlg.h : header file
//

#pragma once

#define APP_NAME "ArtUs RF Data Control"
#define APP_VERSION "1.9.3"

#ifdef _WIN64
#define APP_BITS "(64-bit)"
#else
#define APP_BITS "(32-bit)"
#endif

#include <initguid.h>
#include <dshow.h>
#include "Qedit_custom.h"
#include <windows.h>
#include <ks.h>
#include <ksproxy.h>

#include "strmif.h"
#include "usgfw.h"
#include "usgfweeprom.h"
#include "Usgfw2_h.h"
#include "afxwin.h"
#include "usgdatarf.h"
#include "usgstrrf.h"

#include "usgstr.h"
#include "rgpropid.h"
#include "ProbeEeprom.h"
#include "usgdata.h"
#include "usgfwex.h"

#include <iostream>
#include <fstream>

#include "HidAPI.h"

class CsampleDlg;

//--------------------------------------------------------------------------

class SampleGrabberCallback : public ISampleGrabberCB
{
public:
	SampleGrabberCallback();
	~SampleGrabberCallback();
	void SetUsgWnd(CsampleDlg* wnd, int x, int y);
	void SetPlugin(ISampleGrabber* plugin);

private:
	ISampleGrabber* m_plugin;
	CsampleDlg* m_usg_wnd;
	int m_usg_x;
	int m_usg_y;

	AM_MEDIA_TYPE *m_pMediaType;
public:

	// begin: IUnknown
	virtual HRESULT __stdcall QueryInterface(REFIID riid, void **ppObj);
	virtual ULONG STDMETHODCALLTYPE AddRef(void);
	virtual ULONG STDMETHODCALLTYPE Release(void);
	long m_nRefCount;
	// end: IUnknown

	// begin: ISampleGrabberCB
	STDMETHODIMP BufferCB(double SampleTime, BYTE *pBuffer, long BufferLen);
	STDMETHODIMP SampleCB(double SampleTime, IMediaSample *pSample);

	// end: ISampleGrabberCB

}; // class SampleGrabberCallback

//--------------------------------------------------------------------------

//--------------------------------------------------------------------------

class ScanConverterCallback : public IUsgfwScanConverterPluginCB
{
public:
	ScanConverterCallback();
	~ScanConverterCallback();
	void SetUsgWnd(HWND wnd, int x, int y);
	void SetPlugin(IUsgfwScanConverterPlugin* plugin);

private:
	IUsgfwScanConverterPlugin* m_plugin;
	HWND m_usg_wnd;
	int m_usg_x;
	int m_usg_y;

public:

	// begin: IUnknown
	virtual HRESULT __stdcall QueryInterface(REFIID riid, void **ppObj);
	virtual ULONG STDMETHODCALLTYPE AddRef(void);
	virtual ULONG STDMETHODCALLTYPE Release(void);
	long m_nRefCount;
	// end: IUnknown

	// begin: IUsgfwScanConverterPluginCB

		// receives pointers to input and output media samples
	STDMETHOD(SampleCB) (THIS_
		IMediaSample *pSampleIn,
		IMediaSample *pSampleOut,
		int nOutX1,
		int nOutY1,
		int nOutX2,
		int nOutY2
		) {
		return S_OK;
	}

	// receives pointers to input and output sample buffers
	STDMETHOD(BufferCB) (THIS_
		PBYTE pBufferIn,
		int nInBufferLen,
		PBYTE pBufferOut,
		int nOutBufferLen,
		int nOutX1,
		int nOutY1,
		int nOutX2,
		int nOutY2
		);

	// receives pointers to input and intermediate sample buffers
	STDMETHOD(InInterimBufferCB) (THIS_
		PBYTE pBufferIn,
		int nInBufferLen,
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		int nOutX1,
		int nOutY1,
		int nOutX2,
		int nOutY2
		) {
		return S_OK;
	}

	// receves pointers to input media sample and intermediatesample buffer
	STDMETHOD(InInterimSampleCB) (THIS_
		IMediaSample *pSampleIn,
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		int nOutX1,
		int nOutY1,
		int nOutX2,
		int nOutY2
		) {
		return S_OK;
	}

	// receives pointers to output and intermediate sample buffers
	STDMETHOD(InterimOutBufferCB) (THIS_
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		PBYTE pBufferOut,
		int nOutBufferLen,
		int nOutX1,
		int nOutY1,
		int nOutX2,
		int nOutY2
		) {
		return S_OK;
	}

	// receives pointers to output media sample and intermediate sample buffer
	STDMETHOD(InterimOutSampleCB) (THIS_
		PBYTE pBufferInterim,
		int nInterimBufferLen,
		IMediaSample *pSampleIn,
		int nOutX1,
		int nOutY1,
		int nOutX2,
		int nOutY2
		) {
		return S_OK;
	}

	// receives conversion parameter change pin index
	// if parameter is negative parameter was changed by some filter interface
	STDMETHOD(ParameterCB) (THIS_
		int nPin
		) {
		return S_OK;
	}

	// end: IUsgfwScanConverterPluginCB

}; // class ScanConverterCallback

//--------------------------------------------------------------------------

#define WM_MESSAGE_UPDATE_BITMAP	(WM_APP + 1)
#define WM_MESSAGE_PROBE_ARRIVE		(WM_APP + 2)
#define WM_MESSAGE_PROBE_REMOVE		(WM_APP + 3)
#define WM_MESSAGE_HID_FREEZE		(WM_APP + 4)
#define WM_MESSAGE_FREEZE_AFTER_SCANNING	(WM_APP + 5)

class CsampleDlg : public CDialog, public IUsgCtrlChangeCommon, public IUsgDeviceChangeSink, public CHidManager
{
public:
	CsampleDlg(CWnd* pParent = NULL);	// standard constructor
	~CsampleDlg();

	LONG number_of_RF_rows;


	// Dialog Data
	enum { IDD = IDD_SAMPLE_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

protected:
	HICON m_hIcon;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	bool first_painted;
	void ShowWaitCursor();
	void ShowNormalCursor();

	DECLARE_MESSAGE_MAP()

private:

	int img_w, img_h;								// image width and height
	HWND usg_wnd;									// ultrasound output window
	IConnectionPoint* m_usg_device_change_cpnt;
	DWORD m_usg_device_change_cpnt_cookie;
	IUsgfw2* usgfw2;								// main library interface
	IUsgDataView* data_view;						// data view from probe
	IProbe2* probe;									// current probe
	CComBSTR probe_display_name_bstr;				// device path
	IUsgMixerControl* mixer_control;				// B mixer control
	IUsgImageProperties* image_properties_b;		// B image properties
	IUsgScanLineProperties2* scan_line_properties_b;	// B scan line properties
	IUsgWindowRF* m_pUsgWindowRF;					// RF (radio frequency) data window
	IUsgQualProp* m_pUsgQualProp;					// interface for B mode FPS (frames per second)
	IUsgCineStream* m_pUsgCine;						// cine control interface
	IUsgCineStream* m_pUsgCineB;					// B mode cine control interface
	IUsgCineStream* m_pUsgCineRF;					// RF mode cine control interface
	IUsgCineSink* m_pUsgCineSink;					// cine buffer control interface
	IUsgDataSourceRF* m_pUsgDataSourceRF;			// interface to set desired RF (radio frequency) data type
	ISampleGrabberFilter* sample_grabber_filter;	// sample grabber control for desired stream (B mode RF) from that we get plugin interface
	ISampleGrabber* m_sample_grabber_plugin;		// plugin to that we set callback
	SampleGrabberCallback* m_sample_grabber_callback;	// sample grabber calback where we receive RF data

	IUsgScanType*			scan_type_ctrl;			// Scanning type (standard/trapezoid/compound)
	IUsgDepth*				depth_ctrl;				// Depth
	IUsgViewArea*			view_area_ctrl;			// View Area
	IUsgGain*				b_gain_ctrl;			// B Gain
	IUsgPower*				b_power_ctrl;			// B Power
	IUsgProbeFrequency3*	b_frequency_ctrl;		// B Frequency
	IUsgLineDensity*		b_lines_density_ctrl;	// B Lines Density
	IUsgFocus*				b_focus_ctrl;			// B Focus
	IUsgImageOrientation*	b_image_orientation_ctrl;	// B Image Orientation
	IUsgSteeringAngle*		b_steering_angle_ctrl;	// B Steering Angle angle (for linear probe)
	IUsgViewAngle*			wideview_angle_ctrl;	// WideView angle
	IUsgCompoundAngle*		compound_angle_ctrl;	// Compound angle
	IUsgCompoundFrames*		compound_frames_ctrl;	// Compound frames number

	IConnectionPoint* usg_control_change_cpnt;	// connection point for notifications about changed control value
	DWORD usg_control_change_cpnt_cookie;		// cookie that is used to unadvise connection point

	IUsgScanConverterPlugin* scp;
	IUsgfwScanConverterPlugin* m_plugin;
	ScanConverterCallback* m_scan_converter_callback;

	LONG m_nLineIndex;		// the index of RF line inside RF window

	int m_probe_idx;
	void CreateUsgControl(IUsgDataView* data_view, const IID& type_id, ULONG scan_mode, ULONG stream_id, void** ctrl);
	void CreateUsgControls();
	void ReleaseUsgControls(bool final_release);
	void LoadUsgValues(CString fn, bool show_error_messages);
	void SaveUsgValues(CString fn);
	void WriteSettingsInt(CString section, CString key_name, int val, CString fn);
	void WriteSettingsFloat(CString section, CString key_name, float val, CString fn);
	CString GetSettingsFn();

	void InitializeUserInterfaceControls();
	void EnableDisableUserInterfaceControls();
	void EnableCtrl(int id, BOOL enable);

	void DepthUpdateGUI();
	void DepthSetPrevNext(int dir);

	void B_GainUpdateGUI();
	void B_GainSetByIdx(int idx);

	void B_PowerUpdateGUI();
	void B_PowerSetByIdx(int idx);

	void B_FrequencyUpdateGUI();
	void B_FrequencySetPrevNext(int dir);

	void B_LinesDensityUpdateGUI();
	void B_LinesDensitySetPrevNext(int dir);
	void B_LinesDensitySet(LONG val);

	void B_FocusUpdateGUI();
	void B_FocusSetPrevNext(int dir);
	void SetSingleStaticFocus();

	bool B_ChangeScanDirectionGet();
	void B_ChangeScanDirectionUpdateGui();
	int B_RotationAngleGet();

	void B_SteeringAngleUpdateGUI();
	void B_SteeringAngleSetPrevNext(int dir);

	void WideViewAngleUpdateGUI();
	void WideViewAngleSetPrevNext(int dir);

	void CompoundAngleUpdateGUI();
	void CompoundAngleSetPrevNext(int dir);

	bool compound_frames_enabled;
	void CompoundFramesUpdateGUI();
	void CompoundFramesSetPrevNext(int dir);

	int m_nSubFrameIndex;
	void CompoundSubframeCheckRange();
	void CompoundSubframeUpdateGUI();
	void CompoundSubframeSetPrevNext(int dir);

	void ScanTypeUpdateGUI();
	void ScanTypeSet(int id);

	HDC paint_hdc;					// device context for painting on-ultrasound graphics
	HBITMAP paint_hbmp;				// painting bitmap
	CRITICAL_SECTION paint_cs;		// on-ultrasound painting critical section
	void UpdateAlphaBitmap();		// function to update on-ultrasound graphics
	void DrawBImage(HDC& hdc, int x1, int y1, int x2, int y2);		// paint graphics for B mode image

	CRITICAL_SECTION paint_rf_data_cs;	// RF data painting critical section
	int rf_data_source_point_id;		// identifier of RF data source
	INT16* rf_data_i_arr;				// array of I RF data
	void SetRfDataSourceByPointId(int point_id);
	void RfDataSourcePointUpdateGui();

	INT16* rf_data_i_arr_full;		    // array of I RF data + all rows

	CString rf_data_file_name;			// RF data file name
	FILE *rf_data_file_handle;          // RF data file handle

	INT16* rf_data_q_arr;				// array of Q RF data
	int rf_data_i_len;					// the length of I RF data array
	int rf_data_q_len;					// the length of I RF data array
	bool rf_iq_autofit;
	double rf_data_i_scale;				// calculated scale
	double rf_data_q_scale;
	double rf_data_i_scale2;			// additional scale selected from the user interface
	double rf_data_q_scale2;
	void PaintArr(CDC& dc, COLORREF clr, int x1, int y1, int x2, int y2, INT16* arr, int len, double scale);
	void UpdateScale(int x1, int y1, int x2, int y2, INT16* arr, int len, double& scale);
	void UpdateIScaleGUI();
	void UpdateQScaleGUI();
	void ShowHideQScaleControls();

public:
	// IUnknown
	STDMETHODIMP_(ULONG) AddRef();
	STDMETHODIMP_(ULONG) Release();
	STDMETHODIMP QueryInterface(REFIID riid, void** ppv);
	ULONG m_refCount;

public:

	// IDispatch
	virtual HRESULT STDMETHODCALLTYPE GetTypeInfoCount(UINT *pctinfo);
	virtual HRESULT STDMETHODCALLTYPE GetTypeInfo(UINT itinfo, LCID lcid, ITypeInfo** pptinfo);
	virtual HRESULT STDMETHODCALLTYPE GetIDsOfNames(const IID &riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgdispid);
	virtual HRESULT STDMETHODCALLTYPE Invoke(DISPID dispIdMember, const IID &riid, LCID lcid, WORD wFlags, DISPPARAMS *pDispParams, VARIANT *pVarResult, EXCEPINFO *pExcepInfo, UINT *puArgErr);

public:

	// IUsgCtrlChangeCommon
	virtual HRESULT STDMETHODCALLTYPE OnControlChanged(REFIID riidCtrl, ULONG scanMode, LONG streamId, IUsgControl *pControlObj, LONG dispId, LONG flags)
	{
		return S_OK;
	}
	virtual HRESULT STDMETHODCALLTYPE OnControlChangedBSTR(BSTR ctrlGUID, LONG scanMode, LONG streamId, IUsgControl *pControlObject, LONG dispId, LONG flags);

public:

	// IUsgDeviceChangeSink
	virtual HRESULT STDMETHODCALLTYPE OnProbeArrive(IUnknown *pUsgProbe, ULONG *reserved);
	virtual HRESULT STDMETHODCALLTYPE OnBeamformerArrive(IUnknown *pUsgBeamformer, ULONG *reserved);
	virtual HRESULT STDMETHODCALLTYPE OnProbeRemove(IUnknown *pUsgProbe, ULONG *reserved);
	virtual HRESULT STDMETHODCALLTYPE OnBeamformerRemove(IUnknown *pUsgBeamformer, ULONG *reserved);
	virtual HRESULT STDMETHODCALLTYPE OnProbeStateChanged(IUnknown *pUsgProbe, ULONG *reserved);
	virtual HRESULT STDMETHODCALLTYPE OnBeamformerStateChanged(IUnknown *pUsgBeamformer, ULONG *reserved);

public:
	void WriteToLog(CString str);
	void DisplayChangedParam(BSTR ctrlGUID, LONG scanMode, LONG streamId, IUsgControl *pControlObject, LONG dispId, LONG flags);
	void SetRFDataI32(INT16 *arr, int len);
	void SetRFDataI16(INT16 *arr, int len);
	void SetRFDataQ32(INT16 *arr, int len);
	void SetRFDataI32_full_frame(PULTRASOUNDRF_INFO pFrameInfo, PBYTE pBuffer, int flag, int nSubFrameIndex);
	void PaintRFData();

	void ProcessSample(PBYTE pBuffer, int nBufferLen, PULTRASOUNDRF_INFO pFrameInfo, int nSubFrameIndex, bool updateGUI);

public:
	void FreezeRun();
	afx_msg void OnBnClickedFreezeButton();
	afx_msg void OnBnClickedDepthPrevButton();
	afx_msg void OnBnClickedDepthNextButton();
	afx_msg void OnBnClickedBFrequencyPrevButton();
	afx_msg void OnBnClickedBFrequencyNextButton();
	afx_msg void OnBnClickedBLinesDensityPrevButton();
	afx_msg void OnBnClickedBLinesDensityNextButton();
	afx_msg void OnBnClickedBFocusPrevButton();
	afx_msg void OnBnClickedBFocusNextButton();
	afx_msg void OnBnClickedSteeringAnglePrevButton();
	afx_msg void OnBnClickedSteeringAngleNextButton();
	afx_msg void OnBnClickedWideviewAnglePrevButton();
	afx_msg void OnBnClickedWideviewAngleNextButton();
	afx_msg void OnBnClickedCompoundAnglePrevButton();
	afx_msg void OnBnClickedCompoundAngleNextButton();
	afx_msg void OnBnClickedCompoundFramesPrevButton();
	afx_msg void OnBnClickedCompoundFramesNextButton();
	afx_msg void OnBnClickedCompoundSubframePrevButton();
	afx_msg void OnBnClickedCompoundSubframeNextButton();

	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnClose();
	afx_msg BOOL OnQueryEndSession();
	afx_msg void OnEndSession(BOOL bEnding);
	afx_msg void OnBnClickedRadioAfterSum();

	afx_msg void OnBnClickedRadioWriteRf();
	afx_msg void OnBnClickedRadioStopRf();

	afx_msg void OnBnClickedRadioAfterTfc();
	afx_msg void OnBnClickedRadioAfterApodization();
	afx_msg void OnBnClickedRadioAfterHilbert();
	afx_msg void OnBnClickedButtonPositionUp();
	afx_msg void OnBnClickedButtonPositionDown();
	afx_msg void OnBnClickedButtonPositionLeft();
	afx_msg void OnBnClickedButtonPositionRight();
	afx_msg void OnBnClickedButtonSizeUp();
	afx_msg void OnBnClickedButtonSizeDown();
	afx_msg void OnBnClickedButtonSizeLeft();
	afx_msg void OnBnClickedButtonSizeRight();
	afx_msg void OnBnClickedLinePrevButton();
	afx_msg void OnBnClickedLineNextButton();
	void RF_LineMove(int dx);
	void UpdateLineIndicator();
	afx_msg void OnBnClickedIScalePrevButton();
	afx_msg void OnBnClickedIScaleNextButton();
	afx_msg void OnBnClickedQScalePrevButton();
	afx_msg void OnBnClickedQScaleNextButton();
	afx_msg LRESULT OnUpdateBitmapMessage(WPARAM w, LPARAM l);
	afx_msg LRESULT OnProbeArriveMessage(WPARAM w, LPARAM l);
	afx_msg LRESULT OnProbeRemoveMessage(WPARAM w, LPARAM l);
protected:
	void RF_WindowMove(int dx, int dy);
	void RF_WindowSize(int dx, int dy);
	void RF_WindowUpdatePosition();

public:
	afx_msg void OnTimer(UINT_PTR nIDEvent);		// timer to update FPS indicators
	void StartTimer(void);
	void StopTimer(void);
	int m_nTimer;
	LARGE_INTEGER fps_time;

	int m_nFramesNumberRF;		// the nummber of received RF samples in one second
	int number_of_KBs;          // size of RF file
	int m_nFramesDrawnB;		// the number of drawn B frames in one second
	LONGLONG m_nFramesScannedB;		// the number of scanned B frames in one second
	LONGLONG m_nFramesScannedRF;	// the number of scanned RF frames in one second
	bool check_for_stop;        // to check if RF writing is turned of
	bool do_stop;
	bool record_single_frame;   // set to true if need to record single RF frame

	CStatic m_lblFpsB;			// label for B mode FPS
	CStatic m_lblFpsRF;			// label for RF FPS
	CStatic RFFileSize;			// label for RF file size

	afx_msg void OnBnClickedCinePrev();
	afx_msg void OnBnClickedCinePlay();
	afx_msg void OnBnClickedCineStop();
	afx_msg void OnBnClickedCineNext();
	void UpdateCineFrameIndicator();

	afx_msg void OnBnClickedSaveSettingsToFile();
	afx_msg void OnBnClickedLoadSettingsFromFile();

	bool wideview_is_supported;
	bool compound_is_supported;
	afx_msg void OnBnClickedRadioBStandard();
	afx_msg void OnBnClickedRadioBWideview();
	afx_msg void OnBnClickedRadioBCompound();

	void StartRfWrite();
	void StopRfWrite();

	afx_msg void OnBnClickedRecordSingleFrameButton();

	// begin: HID
private:
	int m_vid;
	int m_pid;
	int m_usage_page;
	int m_usage;
	CHidDevice* m_hid_device; // HID device that we use
	bool m_bUseOverlapped;

private:
	void InitializeHid();
	int GetIntFromHexStr(CString str);
	void SetHidDevice();
	static void HIDCallbackFunction(CsampleDlg *context, CHidDevice *pHidDevice, UINT uHidCode);
	void StartReading();
	void StopReading();
	bool ReadReport(HWND output_wnd);
	bool m_read_in_progress;
	CWinThread* m_read_thread;
	static UINT ReadThreadProc(LPVOID pParam);
	OVERLAPPED m_overlapped;
	volatile LONG m_thread_stop;
	void HandleReceivedData(HWND output_wnd);
	afx_msg LRESULT OnHidFreeze(WPARAM wParam, LPARAM lParam);

	// end: HID

	// begin: line registers
	CComQIPtr<IKsControl> m_iKsControl;
	bool using_regs_line_mode;
	int regs_line1_idx;
	int regs_line2_idx;
	int regs_lines_num;

	void RegsReset();
	void RegsAttach();
	void RegsWrite();
	void RegsClose();
	int RegsGetLine1();
	int RegsGetLine2();
	void RegsSetWindowWidth();
	void RegsUpdateLinesNum();
	// end: line registers

public:
	afx_msg void OnBnClickedProbeNameButton();

	bool record_n_frames;
	int number_of_frames_to_record;
	int recorded_n_frames;
	afx_msg void OnBnClickedRecordNFramesButton();
	CEdit m_record_n_frames_edit;
	void RecordNFramesUpdateGUI();

	void UpdateAutofitGUI();
	afx_msg void OnBnClickedAutofitCheck();

	int display_range;
	afx_msg void OnBnClickedDisplayRangePrevButton();
	afx_msg void OnBnClickedDisplayRangeNextButton();
	void DisplayRangeUpdateGUI();

	void OptionsSave();
	void OptionsLoad();
	CString OptionsGetFn();

	CEdit m_edtCtrl1;
	CEdit m_edtCtrl2;
	afx_msg void OnBnClickedRegsWriteButton();

	CEdit go_to_frame_edit_ctrl;
	afx_msg void OnBnClickedGoToFrameButton();

	void UpdateFrameSliderRange();
	void UpdateFrameSliderCurrent();
	void FrameSliderGoToFrame(int idx);

	CButton freeze_after_scanning_check;
	CEdit freeze_after_scanning_edit;
	afx_msg LRESULT OnFreezeAfterScanning(WPARAM wParam, LPARAM lParam);

	CEdit save_from_edit;
	CEdit save_to_edit;
	afx_msg void OnBnClickedSaveButton();
	afx_msg void OnBnClickedOptionsButton();

	int cine_size_mb;

	CButton m_change_scan_direction_ctrl;
	bool change_scan_direction_updating_gui;
	afx_msg void OnBnClickedChangeScanDirectionCheck();

	CComBSTR GetProbeDisplayNameBstr(int idx);

}; // class CsampleDlg

#define SAFE_RELEASE(x) { if (x) x->Release(); x = NULL; }
